﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;

using System.Web; //Add reference to System.Web
using System.Runtime.Serialization.Json; //Add reference to System.Runtime.Serialization

namespace RedCritterConnecter.Samples
{
    //Create a class to contain the response

    public class GetUserLeaderboardResponse
    {
        public Int64 UserID { get; set; }
        public String TeamID { get; set; }
        public String ProfileEntity { get; set; }
        public Int64 AppID { get; set; }
        public Int64 AppDomainID { get; set; }
        public Boolean Result { get; set; }
        private Int32 ErrorCode { get; set; }
        public Int32 RewardPoints { get; set; }
        public String APIVersion { get; set; }
        public String ErrorMessage { get; set; }
        public Int32 AuthErrorCode { get; set; }
        public List<LeaderboardUser> Users { get; set; }
    }

    public class LeaderboardUser
    {
        public String Email { get; set; }
        public String Username { get; set; }
        public Int64 UserID { get; set; }
        public String FirstName { get; set; }
        public String LastName { get; set; }
        public String IconURL { get; set; }
        public Int32 Rank { get; set; }
        public Double Points { get; set; }
        public Boolean OnBoard { get; set; }
      }

    public class GetUserLeaderboard
    {

        public delegate void OnGetUserLeaderboardResponse(GetUserLeaderboardResponse GetUserLeaderboardResponse);
        public event OnGetUserLeaderboardResponse onGetUserLeaderboardResponse;

        public delegate void OnGetUserLeaderboardResponseError(GetUserLeaderboardResponse GetUserLeaderboardResponse);
        public event OnGetUserLeaderboardResponseError onGetUserLeaderboardResponseError;

        const string CONST_ConnecterBaseURL = "https://www.redcritterconnecter.com/";

        public void Execute(String SecretKey, String Leaderboard)
        {
            try
            {
                //Create url encoded parameters in query string
                String queryString = "secretkey=" + System.Web.HttpUtility.UrlEncode(SecretKey) + "&leaderboardname="+ System.Web.HttpUtility.UrlEncode(Leaderboard);

                //Create a new instance of a WebClient
                WebClient wc = new System.Net.WebClient();

                //Prevent this request from caching in order to ensure that it is sent to server
                wc.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);

                //Attach an event handler to receive the response
                wc.DownloadStringCompleted += onGetUserLeaderboardResponseReceived;

                //Make the call
                            
                wc.DownloadStringAsync(new Uri(CONST_ConnecterBaseURL + "services/gamificationv1/GetUserLeaderboard?" + queryString, UriKind.Absolute));
            }
            catch
            {
                //Something went wrong communicating with the server
                if (onGetUserLeaderboardResponseError != null)
                {
                    onGetUserLeaderboardResponseError(null);
                }
            }

        }

        private void onGetUserLeaderboardResponseReceived(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                //Create a JSON serializer
                System.Runtime.Serialization.Json.DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(GetUserLeaderboardResponse));
               

                //Copy the string into a memory stream
                MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));

                //Read the stream into an object matching the response's type
                GetUserLeaderboardResponse GetUserLeaderboardResponse = (GetUserLeaderboardResponse)s.ReadObject(ms);

                //Work with the populated response object
                if (GetUserLeaderboardResponse.Result == true)
                {
                    //Request was accepted, raise the success event
                    onGetUserLeaderboardResponse(GetUserLeaderboardResponse);
                }
                else
                {
                    //Something went wrong
                    if (onGetUserLeaderboardResponseError != null)
                    {
                        //Request failed, raise the error event
                        onGetUserLeaderboardResponseError(GetUserLeaderboardResponse);
                    }
                }
            }
            catch
            {
                //Something went wrong
                onGetUserLeaderboardResponseError(null);
            }
        }
    }
}
